-----------------------------------------------------------------------------------------------
-- Client Lua Script for GoTimer
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- GoTimer Module Definition
-----------------------------------------------------------------------------------------------
local GoTimer = {} 
local strFormat = '%02d:%02d'
local tZone = {
	KV = {
		oVar = 1662,
		limit = 40,
		},
	ST = {
		oVar = 271,
		limit = 30,
		},
	SC = {
		oVar = 1220,
		limit = 45,
		},
	SSM = {
		oVar = 1553,
		limit = 75,
		},
	}
local inspect
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function GoTimer:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function GoTimer:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- GoTimer OnLoad
-----------------------------------------------------------------------------------------------
function GoTimer:OnLoad()
    -- load our form file

	self.xmlDoc = XmlDoc.CreateFromFile("GoTimer.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

	inspect = Apollo.GetPackage("Drafto:Lib:inspect-1.2").tPackage
	
	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	Apollo.RegisterEventHandler("VarChange_ZoneName", "OnChangeZoneName", self)
	Apollo.RegisterEventHandler("SubZoneChanged", "OnChangeZoneName", self)
end

-----------------------------------------------------------------------------------------------
-- GoTimer OnDocLoaded
-----------------------------------------------------------------------------------------------
function GoTimer:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "TimerWindow", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("gt", "OnGoTimerOn", self)
		self.timer = ApolloTimer.Create(1.0, true, "OnSecond", self)
		self.timer:Stop()
		self.txtBox = self.wndMain:FindChild("TimerBox")
		if self.tElapsed == nil then
			self.tElapsed = {}
		end
		if self.running then
			self:OnResume()
		end
		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- GoTimer Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here


function GoTimer:OnSave(eType)
	return	{
		running = self.running,
		time = tostring(self.time),
		min = self.tElapsed.min and self.tElapsed.min or 0,
		sec = self.tElapsed.sec and self.tElapsed.sec or 0,
		limit = self.limit,
	}
end

function GoTimer:OnRestore(eType, tSavedData)
	self.tElapsed = {}
	self.running = false
	self.saved = tSavedData
	if tSavedData then
		self.tElapsed.min = tSavedData.min
		self.tElapsed.sec = tSavedData.sec
		self.limit = tSavedData.limit
		self.etime = tSavedData.time
		self.running = tSavedData.running
		if self.running then
			local sec =	string.sub(string.match(self.etime, "%d+:%d+:%d+"), -2)
			local sec2 = string.sub(string.match(tostring(Time.Now()), "%d+:%d+:%d+"), -2)
			self.diff = sec2 - sec
			self.tElapsed.sec = self.tElapsed.sec + self.diff
			if self.tElapsed.sec > 59 then
				self.tElapsed.min = self.tElapsed.min + 1
				self.tElapsed.sec = self.tElapsed.sec - 60
			end
		end			
	end
end

function GoTimer:OnChangeZoneName(oVar, strNewZone)
	for key, value in pairs(tZone) do
		if value.oVar and value.oVar == oVar then
			self.limit = value.limit
			self:OnReset()
			self:OnStart()
			self.running = true
			break
		end
	end
end
-- on SlashCommand "/gt"
function GoTimer:OnGoTimerOn()
	if self.running then
		self:OnReset()
	else
		self:OnStart()
	end
	self.running = not self.running
end

function GoTimer:OnStart()
	self.wndMain:Invoke() -- show the window
	self.timer:Start()
	self.tElapsed.min = 0
	self.tElapsed.sec = 0
	self.txtBox:SetText(string.format(strFormat, self.tElapsed.min, self.tElapsed.sec))
end

function GoTimer:OnResume()
	self.wndMain:Invoke() -- show the window
	self.timer:Start()
	self.txtBox:SetText(string.format(strFormat, self.tElapsed.min, self.tElapsed.sec))
end
	
function GoTimer:OnReset()
	self.timer:Stop()
	self.txtBox:SetText("Off")
	self.wndMain:Close()	
end

function GoTimer:OnSecond()
	self.time = Time.Now()
	self.tElapsed.sec = self.tElapsed.sec + 1
	if self.tElapsed.sec == 60 then
		self.tElapsed.min = self.tElapsed.min + 1
		self.tElapsed.sec = 0
	end
	self.txtBox:SetText(string.format(strFormat, self.tElapsed.min, self.tElapsed.sec))
end

function GoTimer:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "GoTimer"})
end
-----------------------------------------------------------------------------------------------
-- GoTimerForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function GoTimer:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function GoTimer:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- GoTimer Instance
-----------------------------------------------------------------------------------------------
local GoTimerInst = GoTimer:new()
GoTimerInst:Init()
